/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol". 
 */

package com.funambol.syncml.client;

import jmunit.framework.cldc10.TestCase;

import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.SyncManager;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.syncml.spds.SourceConfig;

import com.funambol.storage.NamedObjectStore;
import com.funambol.util.Log;

public class SyncTest extends TestCase {

    private static final String STORE_NAME = "TESTCONFIG";
    private static final String SOURCE_NAME = "source.briefcase";
    private static final String URL  = "http://localhost:8080/funambol/ds";
    
    private SourceConfig sc;
    private SyncConfig conf;

    public SyncTest() {
        super(6, "SyncTest");
        
        Log.setLogLevel(Log.DEBUG);

        NamedObjectStore os = new NamedObjectStore();
        sc = new SourceConfig();
        conf = new SyncConfig();

        try {
            os.create(STORE_NAME);
            // Set defaults
            conf.syncUrl = URL;
            os.store(SyncConfig.NAME, conf);

            // Set defaults
            sc.setType("text/plain");
            sc.setEncoding(SyncSource.ENCODING_NONE);
            os.store(SOURCE_NAME, sc);
        }
        catch (Exception e) {
            e.printStackTrace();
            Log.error("Can't init config.");
        }
    }
    
    public void setUp() {
    }

    public void tearDown() {
    }
    
    public void test(int testNumber) throws Throwable {

        NamedObjectStore os = new NamedObjectStore();
        conf = new SyncConfig();
        sc = new SourceConfig();

        try {
            os.open(STORE_NAME);
            os.retrieve(SyncConfig.NAME, conf);
            os.retrieve(SOURCE_NAME, sc);
        }
        catch(Exception e) {
            Log.debug("Error retrieving conf: "+e.toString());
            // Set defaults
            conf.syncUrl = URL;
            sc.setType("text/plain");
            sc.setEncoding(SyncSource.ENCODING_NONE);
        }

        SyncManager sm = new SyncManager(conf);
        TestSyncSource testsrc = new TestSyncSource(sc);

        switch(testNumber) {
            case 0: 
                testSlow(sm, testsrc);
                break;
            case 1: 
                testFast(sm, testsrc);
                break;
            case 2: 
                testOneWayFromClient(sm, testsrc);        
                break;
            case 3: 
                testOneWayFromServer(sm, testsrc);        
                break;
            case 4: 
                testRefreshFromClient(sm, testsrc);
                break;
            case 5: 
                testRefreshFromServer(sm, testsrc);
                break;
            default:                    
                break;
        }

        // Save Anchors
        os.store(SOURCE_NAME, sc);

        os.close();
        os = null;
    }

    public void testSlow(SyncManager sm, SyncSource src) throws Exception {
        Log.info("------------ BEGIN TEST SLOW --------------");
        sm.sync(src, SyncML.ALERT_CODE_SLOW);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST SLOW ---------------");
    }

    public void testFast(SyncManager sm, SyncSource src) throws Exception {
        Log.info("------------ BEGIN TEST FAST --------------");
        sm.sync(src);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST FAST --------------");
    }

    public void testOneWayFromClient(SyncManager sm, SyncSource src)
    throws Exception {
        Log.info("------------ BEGIN TEST ONE WAY FROM CLIENT --------------");
        sm.sync(src, SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST ONE WAY FROM CLIENT --------------");
    }

    public void testOneWayFromServer(SyncManager sm, SyncSource src)
    throws Exception {
        Log.info("------------ BEGIN TEST ONE WAY FROM SERVER --------------");
        sm.sync(src, SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST ONE WAY FROM SERVER --------------");
    }

    public void testRefreshFromClient(SyncManager sm, SyncSource src)
    throws Exception {
        Log.info("------------ BEGIN TEST REFRESH FROM CLIENT --------------");
        sm.sync(src, SyncML.ALERT_CODE_REFRESH_FROM_CLIENT);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST REFRESH FROM CLIENT --------------");
    }
    
    public void testRefreshFromServer(SyncManager sm, SyncSource src)
    throws Exception {
        Log.info("------------ BEGIN TEST REFRESH FROM SERVER --------------");
        sm.sync(src, SyncML.ALERT_CODE_REFRESH_FROM_SERVER);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST REFRESH FROM SERVER --------------");
    }

}

